import sys

from google.protobuf import text_format
from foo import bar_pb2
from com_envoyproxy_protoc_gen_validate.python.protoc_gen_validate import validator
from typing import List


def main(filenames: List[str]) -> None:
    if not filenames:
        print("No inputs provided; exiting")

    success_count = 0
    for filename in filenames:
        bars = bar_pb2.Bars()
        fullname = bars.DESCRIPTOR.name
        try:
            with open(filename, 'r') as fh:
                text_format.Parse(fh.read(), bars)
        except IOError as error:
            msg = f"Failed to open file '{filename}': {error}"
            raise RuntimeError(msg) from error
        except text_format.ParseError as error:
            msg = f"Failed to parse file '{filename}' as a {fullname} textproto: {error}"
            raise RuntimeError(msg) from error

        try:
            validator.validate(bars)
        except validator.ValidationFailed as error:
            print(
                f"Failed to validate file '{filename}' as a {fullname} textproto: {error}",
                file=sys.stderr,
            )
        else:
            print(f"Successfully validated file '{filename}' as a {fullname} textproto")
            success_count += 1

    failure_count = len(filenames) - success_count
    if failure_count:
        s = "s" if failure_count > 1 else ""
        msg = f"Failed to validate {failure_count} file{s}"
        raise RuntimeError(msg)


if __name__ == "__main__":
    try:
        main(sys.argv[1:])
    except RuntimeError as error:
        print(error, file=sys.stderr)
        sys.exit(1)



import sys
import inspect

from python.protoc_gen_validate.validator import validate, validate_all, ValidationFailed

from tests.harness.harness_pb2 import TestCase, TestResult
from tests.harness.cases.bool_pb2 import *
from tests.harness.cases.bytes_pb2 import *
from tests.harness.cases.enums_pb2 import *
from tests.harness.cases.enums_pb2 import *
from tests.harness.cases.messages_pb2 import *
from tests.harness.cases.numbers_pb2 import *
from tests.harness.cases.oneofs_pb2 import *
from tests.harness.cases.repeated_pb2 import *
from tests.harness.cases.strings_pb2 import *
from tests.harness.cases.maps_pb2 import *
from tests.harness.cases.wkt_any_pb2 import *
from tests.harness.cases.wkt_duration_pb2 import *
from tests.harness.cases.wkt_nested_pb2 import *
from tests.harness.cases.wkt_wrappers_pb2 import *
from tests.harness.cases.wkt_timestamp_pb2 import *
from tests.harness.cases.kitchen_sink_pb2 import *

message_classes = {}
for k, v in inspect.getmembers(sys.modules[__name__], inspect.isclass):
    if 'DESCRIPTOR' in dir(v):
        message_classes[v.DESCRIPTOR.full_name] = v


if __name__ == "__main__":
    read = sys.stdin.buffer.read()

    testcase = TestCase()
    testcase.ParseFromString(read)

    test_class = message_classes[testcase.message.TypeName()]
    test_msg = test_class()
    testcase.message.Unpack(test_msg)

    try:
        result = TestResult()
        valid = validate(test_msg)
        result.Valid = True
    except ValidationFailed as e:
        result.Valid = False
        result.Reasons[:] = [repr(e)]

    try:
        result_all = TestResult()
        valid = validate_all(test_msg)
        result_all.Valid = True
    except ValidationFailed as e:
        result_all.Valid = False
        result_all.Reasons[:] = [repr(e)]

    if result.Valid != result_all.Valid:
        raise ValueError(f"validation results mismatch, validate: {result.Valid}, "
                         f"validate_all: {result_all.Valid}")
    if not result.Valid:
        reason = list(result.Reasons)[0]  # ValidationFailed("reason")
        reason = reason[18:-2]  # reason
        reason_all = list(result_all.Reasons)[0]  # ValidationFailed("reason1\nreason2\n...reason")
        reason_all = reason_all[18:-2]  # reason1\nreason2\n...reason
        if not reason_all.startswith(reason):
            raise ValueError(f"different first message, validate: {reason}, "
                             f"validate_all: {reason_all}")

    sys.stdout = open(sys.stdout.fileno(), mode='w', encoding='utf8')
    sys.stdout.write(result.SerializeToString().decode("utf-8"))



import configparser

# There's two sets of requirements relevant for python. The first set in requirements.txt is installed
# during the Docker build and is used for linting, building, and uploading the PGV python package to PyPI.
#
# The other set is in the install_requires section of setup.cfg. This is what's needed to use the package.
#
# We use pip_install from @rules_python to install these requirements in order to test the package. Unfortunately:
# - pip_install can't handle setup.cfg directly, it wants a file containing a simple list
# - as a bazel repository_rule, pip_install won't accept generated files as input so we can't autogen
#   this simpler file out of setup.cfg as part of bazel build.
#
# So instead here we just check that requirements.in matches what's in install_requires of setup.cfg.


with open('python/requirements.in', 'r') as reqs:
    lines = reqs.readlines()
    requirements_dot_in_set = {line.strip() for line in lines if line.strip() and not line.startswith("#")}

config = configparser.ConfigParser()
config.read('python/setup.cfg')
setup_dot_cfg_set = {line for line in config['options']['install_requires'].split() if not line.startswith("#")}

assert requirements_dot_in_set == setup_dot_cfg_set
